use serenity::{
    all::Message,
    builder::{CreateEmbed, CreateEmbedFooter, CreateMessage},
    client::Context,
    framework::standard::{macros::command, Args, CommandResult},
    model::colour,
};

use crate::db::user_image::UserImage;

#[command]
#[description = "Enable/Disable the auto meme answer"]
pub async fn enable(ctx: &Context, msg: &Message, mut args: Args) -> CommandResult {
    let guild = match msg.guild_id {
        Some(guild) => guild,
        None => return Ok(()),
    };
    let footer = CreateEmbedFooter::new("WeeboBot");
    let embed = CreateEmbed::new()
        .title("Enable/Disable auto meme answer")
        .footer(footer);
    let mut user_in_db: UserImage =
        match UserImage::find_by_server_id_user_id(&guild.get(), &msg.author.id.get()).await {
            Ok(Some(user_in_db)) => user_in_db.clone(),
            Ok(None) => {
                let user_in_db = UserImage::new(guild.get(), msg.author.id.get(), false).unwrap();
                match user_in_db.create().await {
                    Ok(_) => user_in_db,
                    Err(e) => {
                        let embed = embed
                            .field("Could't create user in db", e.to_string(), false)
                            .color(colour::Color::RED);
                        let builder = CreateMessage::new().embed(embed).reference_message(msg);
                        if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
                            println!("Error sending message: {:?}", why)
                        }
                        println!("Error saving user image: {:?}", e);
                        return Ok(());
                    }
                }
            }
            Err(e) => {
                let embed = embed
                    .field("Error finding user image", e.to_string(), false)
                    .color(colour::Color::RED);
                let builder = CreateMessage::new().embed(embed).reference_message(msg);
                if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
                    println!("Error sending message: {:?}", why)
                }
                println!("Error finding user image: {:?}", e);
                return Ok(());
            }
        };
    if let Ok(new_status) = args.single::<bool>() {
        if user_in_db.enable == new_status {
            let embed = embed.field(
                format!("Auto answer the same : {}", new_status),
                String::new(),
                false,
            );
            let builder = CreateMessage::new().embed(embed).reference_message(msg);
            if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
                println!("Error sending message: {:?}", why)
            }
            return Ok(());
        }
        user_in_db.enable = new_status;
        match user_in_db.update().await {
            Ok(_) => {
                let embed = embed.field("Auto answer", new_status.to_string(), false);
                let builder = CreateMessage::new().embed(embed).reference_message(msg);
                if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
                    println!("Error sending message: {:?}", why)
                }
            }
            Err(e) => {
                let embed = embed.field("Couldn't update user in db", e.to_string(), false);
                let builder = CreateMessage::new().embed(embed).reference_message(msg);
                if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
                    println!("Error sending message: {:?}", why)
                }
                println!("Error saving user image: {:?}", e);
                return Ok(());
            }
        }
        return Ok(());
    } else {
        let embed = embed.field("Auto answer", user_in_db.enable.to_string(), false);
        let builder = CreateMessage::new().embed(embed).reference_message(msg);
        if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
            println!("Error sending message: {:?}", why)
        }
    }

    Ok(())
}
