use serenity::builder::{CreateEmbed, CreateEmbedFooter, CreateMessage};
use serenity::{
    all::Message,
    client::Context,
    framework::standard::{macros::command, CommandResult},
};

use crate::img::config_file::ConfigImgGlobal;

#[command]
#[description = "List supported meme keywords"]
pub async fn list(ctx: &Context, msg: &Message) -> CommandResult {
    let config_img = {
        let data_read = ctx.data.read().await;
        data_read
            .get::<ConfigImgGlobal>()
            .expect("Config img not found")
            .clone()
    };
    let list_value: Vec<(String, String, bool)> = config_img
        .keyword
        .iter()
        .map(|x| (x.value.clone().join(", "), String::new(), true))
        .collect();
    if list_value.is_empty() {
        let builder = CreateMessage::new().content("No meme keyword found");
        if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
            println!("Error sending message: {:?}", why)
        }
        return Ok(());
    }
    for chunks in list_value.chunks(25) {
        let footer = CreateEmbedFooter::new("WeeboBot");
        let embed = CreateEmbed::new()
            .title("Meme List")
            .fields(chunks.to_vec())
            .footer(footer);
        let builder = CreateMessage::new().embed(embed);
        if let Err(why) = msg.channel_id.send_message(&ctx.http, builder).await {
            println!("Error sending message: {:?}", why)
        }
    }
    Ok(())
}
