use poise::serenity_prelude as serenity;
use std::fs;
use serenity::GatewayIntents;
use crate::botv2::cmd::{ping::ping,help::help, meme::list::list};
use crate::{botv2::handler::event_handler, img::config_file::ConfigFile};
use crate::config::Config;
use tokio::sync::oneshot;
use tokio::task::spawn_blocking;
pub struct Data{
    pub config_img: ConfigFile,
    pub config: Config,
}

// Types used by all command functions
pub type Error = Box<dyn std::error::Error + Send + Sync>;
pub type Context<'a> = poise::Context<'a, Data, Error>;

#[poise::command(slash_command, prefix_command)]
async fn age(
    ctx: Context<'_>,
    #[description = "Selected user"] user: Option<serenity::User>,
) -> Result<(), Error> {
    let u = user.as_ref().unwrap_or_else(|| ctx.author());
    let response = format!("{}'s account was created at {}", u.name, u.created_at());
    ctx.say(response).await?;
    Ok(())
}



pub fn start_bot(config: Config, rx: oneshot::Receiver<()>){
    if config.token != "" {
        spawn_blocking(move||{
            let rt = tokio::runtime::Handle::current();
            rt.block_on(async move{
                let local = tokio::task::LocalSet::new();
                let _ = local.run_until(async move{
                    let config_img = match fs::read_to_string(format!(
                        "{}/config.yaml",
                        config.image.path
                    )) {
                        Ok(content) => content,
                        Err(err) => {
                            println!("Error while opening config.yaml : {:?}", err);
                            return;
                        }
                    };
                    let config_parsed = match ConfigFile::parse_config(config_img) {
                        Ok(config) => config,
                        Err(err) => {
                            println!("Error while parsing config.yaml : {:?}", err);
                            return;
                        }
                    };
                    let intents = GatewayIntents::GUILD_MESSAGES
                        | GatewayIntents::DIRECT_MESSAGES
                        | GatewayIntents::MESSAGE_CONTENT
                        | GatewayIntents::GUILD_VOICE_STATES
                        | GatewayIntents::GUILDS
                        | GatewayIntents::GUILD_MEMBERS
                        | GatewayIntents::GUILD_PRESENCES
                        | GatewayIntents::GUILD_MESSAGE_REACTIONS;
                    let token = config.token.clone();
                    let prefix = config.prefix.clone();
                    let framework = poise::Framework::builder()
                        .options(poise::FrameworkOptions {
                            commands: vec![age(), ping(),help(), list()],
                            prefix_options: poise::PrefixFrameworkOptions {
                                prefix: Some(prefix.into()),
                                ..Default::default()
                            },
                            event_handler: |ctx, event, framework, data| {
                                Box::pin(event_handler(ctx, event, framework, data))
                            },
                            ..Default::default()
                        })
                        .setup(|ctx, _ready, framework| {
                            Box::pin(async move {
                                poise::builtins::register_globally(ctx, &framework.options().commands).await?;
                                Ok(Data {
                                    config_img: config_parsed,
                                    config: config.clone(),
                                })
                            })
                        })
                        .build();
                    let mut client = serenity::ClientBuilder::new(token, intents)
                        .framework(framework)
                        .await.expect("Error creating client");
                    let shard_manager = client.shard_manager.clone();
                    let client_start = client.start_autosharded();
                    tokio::spawn(async move {
                        match rx.await {
                            Ok(_) => {
                                println!("Received shutdown signal");
                                shard_manager.shutdown_all().await;
                                println!("Shutting down bot");
                            }
                            Err(_) => {
                                println!("Channel dropped signal");
                                shard_manager.shutdown_all().await;
                                println!("Shutting down bot");
                            }
                        }
                    });
                    println!("Bot is running...");
                    if let Err(why) = client_start.await {
                        println!("Client error: {why:?}");
                    }
                    println!("Bot is stopped...");
                }).await;
            })
        });
    }
}